@@ -1,5 +1,5 @@
 /*  Lziprecover - Data recovery tool for the lzip format
-    Copyright (C) 2009-2014 Antonio Diaz Diaz.
+    Copyright (C) 2009-2015 Antonio Diaz Diaz.
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -40,7 +40,7 @@ public:
 
 enum {
   min_dictionary_bits = 12,
-  min_dictionary_size = 1 << min_dictionary_bits,	// >= modeled_distances
+  min_dictionary_size = 1 << min_dictionary_bits,	/* >= modeled_distances */
   max_dictionary_bits = 29,
   max_dictionary_size = 1 << max_dictionary_bits,
   min_member_size = 36,
@@ -53,7 +53,7 @@ enum {
   dis_slot_bits = 6,
   start_dis_model = 4,
   end_dis_model = 14,
-  modeled_distances = 1 << (end_dis_model / 2),		// 128
+  modeled_distances = 1 << (end_dis_model / 2),		/* 128 */
   dis_align_bits = 4,
   dis_align_size = 1 << dis_align_bits,
 
@@ -65,8 +65,8 @@ enum {
   len_high_symbols = 1 << len_high_bits,
   max_len_symbols = len_low_symbols + len_mid_symbols + len_high_symbols,
 
-  min_match_len = 2,					// must be 2
-  max_match_len = min_match_len + max_len_symbols - 1,	// 273
+  min_match_len = 2,					/* must be 2 */
+  max_match_len = min_match_len + max_len_symbols - 1,	/* 273 */
   min_match_len_limit = 5 };
 
 inline int get_len_state( const int len )
@@ -186,9 +186,9 @@ const uint8_t magic_string[4] = { 0x4C, 0x5A, 0x49, 0x50 };	// "LZIP"
 
 struct File_header
   {
-  uint8_t data[6];			// 0-3 magic bytes
-					//   4 version
-					//   5 coded_dict_size
+  uint8_t data[6];			/* 0-3 magic bytes */
+					/*   4 version */
+					/*   5 coded_dict_size */
   enum { size = 6 };
 
   void set_magic() { std::memcpy( data, magic_string, 4 ); data[4] = 1; }
@@ -214,9 +214,9 @@ struct File_header
       if( sz > min_dictionary_size )
         {
         const unsigned base_size = 1 << data[5];
-        const unsigned wedge = base_size / 16;
+        const unsigned fraction = base_size / 16;
         for( int i = 7; i >= 1; --i )
-          if( base_size - ( i * wedge ) >= sz )
+          if( base_size - ( i * fraction ) >= sz )
             { data[5] |= ( i << 5 ); break; }
         }
       return true;
@@ -228,9 +228,9 @@ struct File_header
 
 struct File_trailer
   {
-  uint8_t data[20];	//  0-3  CRC32 of the uncompressed data
-			//  4-11 size of the uncompressed data
-			// 12-19 member size including header and trailer
+  uint8_t data[20];	/*  0-3  CRC32 of the uncompressed data */
+			/*  4-11 size of the uncompressed data */
+			/* 12-19 member size including header and trailer */
 
   enum { size = 20 };
 
@@ -290,12 +290,12 @@ int seek_read( const int fd, uint8_t * const buf, const int size,
 int open_instream( const char * const name, struct stat * const in_statsp,
                    const bool no_ofile, const bool reg_only = false );
 int open_outstream_rw( const std::string & output_filename, const bool force );
-void show_header( const File_header & header );
+void show_header( const unsigned dictionary_size );
 void show_error( const char * const msg, const int errcode = 0,
                  const bool help = false );
+void internal_error( const char * const msg );
 void show_error2( const char * const msg1, const char * const name,
                   const char * const msg2 );
-void internal_error( const char * const msg );
 
 // defined in merge.cc
 void cleanup_and_fail( const std::string & output_filename,
@@ -309,18 +309,19 @@ int merge_files( const std::vector< std::string > & filenames,
                  const bool force );
 
 // defined in range_dec.cc
+const char * format_num( unsigned long long num,
+                         unsigned long long limit = -1ULL,
+                         const int set_prefix = 0 );
 bool safe_seek( const int fd, const long long pos );
 int list_files( const std::vector< std::string > & filenames,
                 const int verbosity );
-int range_decompress( const std::string & input_filename,
-                      const std::string & default_output_filename,
-                      const std::string & range_string, const int verbosity,
-                      const bool force, const bool ignore, const bool to_stdout );
 
 // defined in repair.cc
 int repair_file( const std::string & input_filename,
                  const std::string & output_filename, const int verbosity,
                  const bool force );
+int debug_repair( const std::string & input_filename, const long long bad_pos,
+                  const int verbosity, const uint8_t bad_value );
 
 // defined in split.cc
 int split_file( const std::string & input_filename,
