@@ -1,9 +1,9 @@
-/*  Lziprecover - Data recovery tool for lzipped files
-    Copyright (C) 2009, 2010, 2011, 2012 Antonio Diaz Diaz.
+/*  Lziprecover - Data recovery tool for the lzip format
+    Copyright (C) 2009-2015 Antonio Diaz Diaz.
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
-    the Free Software Foundation, either version 3 of the License, or
+    the Free Software Foundation, either version 2 of the License, or
     (at your option) any later version.
 
     This program is distributed in the hope that it will be useful,
@@ -28,13 +28,15 @@
 #include <sys/stat.h>
 
 #include "lzip.h"
+#include "block.h"
+#include "file_index.h"
 
 
 namespace {
 
 void first_filename( const std::string & input_filename,
                      const std::string & default_output_filename,
-                     std::string & output_filename )
+                     std::string & output_filename, const int max_digits )
   {
   if( default_output_filename.size() )
     output_filename = default_output_filename;
@@ -42,15 +44,16 @@ void first_filename( const std::string & input_filename,
     output_filename = input_filename;
   int b = output_filename.size();
   while( b > 0 && output_filename[b-1] != '/' ) --b;
-  output_filename.insert( b, "rec00001" );
+  output_filename.insert( b, "rec1" );
+  if( max_digits > 1 ) output_filename.insert( b + 3, max_digits - 1, '0' );
   }
 
 
-bool next_filename( std::string & output_filename )
+bool next_filename( std::string & output_filename, const int max_digits )
   {
   int b = output_filename.size();
   while( b > 0 && output_filename[b-1] != '/' ) --b;
-  for( int i = b + 7; i >= b + 3; --i )			// "rec00001"
+  for( int i = b + max_digits + 2; i > b + 2; --i )	// "rec<max_digits>"
     {
     if( output_filename[i] < '9' ) { ++output_filename[i]; return true; }
     else output_filename[i] = '0';
@@ -59,8 +62,27 @@ bool next_filename( std::string & output_filename )
   }
 
 
+bool verify_header( const File_header & header, const Pretty_print & pp )
+  {
+  if( !header.verify_magic() )
+    {
+    pp( "Bad magic number (file not in lzip format)." );
+    return false;
+    }
+  if( !header.verify_version() )
+    {
+    if( pp.verbosity() >= 0 )
+      { pp();
+        std::fprintf( stderr, "Version %d member format not supported.\n",
+                      header.version() ); }
+    return false;
+    }
+  return true;
+  }
+
+
 // Search forward from 'pos' for "LZIP" (Boyer-Moore algorithm)
-// Return pos of found string or 'pos+size' if not found.
+// Returns pos of found string or 'pos+size' if not found.
 //
 int find_magic( const uint8_t * const buffer, const int pos, const int size )
   {
@@ -83,32 +105,41 @@ int find_magic( const uint8_t * const buffer, const int pos, const int size )
 
 
 int do_split_file( const std::string & input_filename, uint8_t * & base_buffer,
-                   const std::string & default_output_filename, const bool force )
+                   const std::string & default_output_filename,
+                   const int verbosity, const bool force )
   {
   const int hsize = File_header::size;
-  const int tsize = File_trailer::size();
+  const int tsize = File_trailer::size;
   const int buffer_size = 65536;
   const int base_buffer_size = tsize + buffer_size + hsize;
   base_buffer = new uint8_t[base_buffer_size];
   uint8_t * const buffer = base_buffer + tsize;
 
   struct stat in_stats;
-  const int infd = open_instream( input_filename, &in_stats, true, true );
+  const int infd = open_instream( input_filename.c_str(), &in_stats, true, true );
   if( infd < 0 ) return 1;
-  int size = readblock( infd, buffer, buffer_size + hsize ) - hsize;
+  Pretty_print pp( input_filename, verbosity );
+  int size = seek_read( infd, buffer, buffer_size + hsize, 0 ) - hsize;
   bool at_stream_end = ( size < buffer_size );
   if( size != buffer_size && errno )
     { show_error( "Read error", errno ); return 1; }
-  if( size <= tsize )
-    { show_error( "Input file is too short." ); return 2; }
-  if( !verify_header( *(File_header *)buffer ) ) return 2;
+  if( size < min_member_size )
+    { pp( "Input file is too short." ); return 2; }
+  if( !verify_header( *(File_header *)buffer, pp ) ) return 2;
+
+  const File_index file_index( infd );
+  if( file_index.retval() != 0 ) pp( file_index.error().c_str() );
+  const long max_members = file_index.retval() ? 999999 : file_index.members();
+  int max_digits = 1;
+  for( long i = max_members; i >= 10; i /= 10 ) ++max_digits;
 
   std::string output_filename;
-  first_filename( input_filename, default_output_filename, output_filename );
+  first_filename( input_filename, default_output_filename, output_filename,
+                  max_digits );
   int outfd = open_outstream_rw( output_filename, force );
   if( outfd < 0 ) { close( infd ); return 1; }
 
-  long long partial_member_size = 0;
+  unsigned long long partial_member_size = 0;
   while( true )
     {
     int pos = 0;
@@ -117,17 +148,20 @@ int do_split_file( const std::string & input_filename, uint8_t * & base_buffer,
       newpos = find_magic( buffer, newpos, size + 4 - newpos );
       if( newpos <= size )
         {
-        long long member_size = 0;
-        for( int i = 1; i <= 8; ++i )
-          { member_size <<= 8; member_size += base_buffer[tsize+newpos-i]; }
-        if( partial_member_size + newpos - pos == member_size )
+        const File_trailer & trailer = *(File_trailer *)(base_buffer + newpos);
+        if( partial_member_size + newpos - pos == trailer.member_size() )
           {						// header found
           const int wr = writeblock( outfd, buffer + pos, newpos - pos );
           if( wr != newpos - pos )
             { show_error( "Write error", errno ); return 1; }
           if( close( outfd ) != 0 )
             { show_error( "Error closing output file", errno ); return 1; }
-          if( !next_filename( output_filename ) )
+          if( verbosity >= 1 )
+            {
+            std::printf( "Member '%s' done \r", output_filename.c_str() );
+            std::fflush( stdout );
+            }
+          if( !next_filename( output_filename, max_digits ) )
             { show_error( "Too many members in file." ); close( infd ); return 1; }
           outfd = open_outstream_rw( output_filename, force );
           if( outfd < 0 ) { close( infd ); return 1; }
@@ -160,6 +194,11 @@ int do_split_file( const std::string & input_filename, uint8_t * & base_buffer,
   close( infd );
   if( close( outfd ) != 0 )
     { show_error( "Error closing output file", errno ); return 1; }
+  if( verbosity >= 1 )
+    {
+    std::printf( "Member '%s' done \n", output_filename.c_str() );
+    std::fflush( stdout );
+    }
   return 0;
   }
 
@@ -167,11 +206,12 @@ int do_split_file( const std::string & input_filename, uint8_t * & base_buffer,
 
 
 int split_file( const std::string & input_filename,
-                const std::string & default_output_filename, const bool force )
+                const std::string & default_output_filename,
+                const int verbosity, const bool force )
   {
   uint8_t * base_buffer;
   const int retval = do_split_file( input_filename, base_buffer,
-                                    default_output_filename, force );
+                                    default_output_filename, verbosity, force );
   delete[] base_buffer;
   return retval;
   }
